#ifndef EDGE_HH
#define EDGE_HH

/*
  James R. Diebel
  Stanford University
  
  Started: 27 August 2004
  Last revised: 27 Sugust 2004

  edge.hh - class definition for an edge in a 3d surface mesh as defined
  in the header file mesh.hh.

  Depends on:
  - Vec3d class (vec3d.hh)
  - Vert class (vert.hh)
  - Face class (face.hh)

  Used almost exclusively by Mesh class (mesh.hh)
*/

#include <iostream>
#include "vec3d.hh"
#include "mesh.hh"
#include "vert.hh"
#include "face.hh"

namespace bmtk {

using namespace std;

class Edge {
 public:
  int flag; // visitation flag
  int nf; // number of adjacent faces
  int i; // index of self within global edge list
  float psi; // the edge potential
  float a; // max angle between adjacent faces
  float l; // the length of the edge

  class Mesh *m; // link to parent mesh
  class Face *f[2]; // list of adjacent faces
  class Vert *v[2]; // list of vertices of edge

  // Constructor/Destructor
  Edge();
  ~Edge();

  // Member functions
  void findAngle();
  void findLength();
  void findPotential();
  Vec3d gradient(Vert& vi);
  int indexOf(Vert& vp);

  // Equality tests
  bool operator == (const Edge &e) const;
  bool operator != (const Edge &e) const;
};

// Membership tests
bool operator < (Vert &vp, Edge &ep);

// Define << operator to handle output to screen
ostream& operator << (ostream& os, const Edge &e);

} // namespace bmtk

#endif
