/*
  James R. Diebel
  Stanford University

  Started: 27 August 2004
  Last revised: 27 Sugust 2004

  face.cc - class implementation of Face class (face.hh)

  Depends on face.hh
*/

#include "bmtk/face.hh"

namespace bmtk {

// Constructor/Destructor
Face::Face() {
  nf = ne = 0;
  i = -1;
  bound = false;
  flag = -1;
}
Face::~Face() {
  if (m) if (m->nd) delete [] ns;
}

// Member functions
void Face::findNormal() {
  n = (v[1]->x - v[0]->x).cross(v[2]->x - v[0]->x);
  s = 0.5*~n;
  n.normalize();
}

void Face::findCenter() {
  x = (v[0]->x + v[1]->x + v[2]->x)/3;
}

void Face::findDistance() {
  d = x.dot(n);
}

void Face::findArea() {
  s = 0.5*~((v[1]->x - v[0]->x).cross(v[2]->x - v[0]->x));
}

void Face::blur() {
  n = nr;
  for (int j=0;j<nf;j++) n += f[j]->nr;
  n.normalize();
}

int Face::indexOf(Vert& vp) {
  if (v[0] == &vp) return 0;
  if (v[1] == &vp) return 1;
  if (v[2] == &vp) return 2;
  return -1;
}

// Membership tests
bool operator < (Vert &vp, Face &fp) {
  return (fp.v[0] == &vp || fp.v[1] == &vp || fp.v[2] == &vp);
}
bool operator < (Edge &ep, Face &fp) {
  return ((fp.v[0] == ep.v[0] || fp.v[1] == ep.v[0] || fp.v[2] == ep.v[0]) &&
    (fp.v[0] == ep.v[1] || fp.v[1] == ep.v[1] || fp.v[2] == ep.v[1]));
}

// Define << operator to handle output to screen
ostream& operator << (ostream& os, const Face &f) {
  return os<<"f"<<f.i<<": "<<f.v[0]->i<<", "<<f.v[1]->i
     <<", "<<f.v[2]->i<<" ("<<f.nf<<", "<<f.ne<<")"<<endl<<flush;
}

} // namespace bmtk
