/*
  James R. Diebel
  Stanford University

  Started: 9 January 2005

  lst2msh.cc - utility to convert a list of PLY files into a SuperMesh

  Depends on:
  - Mesh class (mesh.hh)
  -- Vec3d class (vec3d.hh)
  -- Vert class (vert.hh)
  -- Edge class (edge.hh)
  -- Face class (face.hh)
  - Timer class (timer.hh)
  - TriMesh class (trimesh.h)
*/


#include <iostream>
#include "bmtk/mesh.hh"
#include "bmtk/timer.hh"
#include <trimesh/trimesh.h>

using namespace std;
using namespace bmtk;

int main (int argc, char *argv[]) {
  // if bad command line args, give usage instructions and exit
  if (argc < 3) {
    cout << endl
	 << "Usage: ply2msh file.lst outfile.msh" << endl
	 << "    file.lst must be a list of ply files" << endl
	 << "    outfile.msh is the name of the output msh file" << endl;
    exit(1);
  }

  // create empty Mesh and then fill it with data from ply file
  Mesh m;
  m.buildFromLST(argv[1]);

  // output it to the specified file
  m.writeMesh(argv[2]);

  return 0;
}
