/*
  James R. Diebel
  Stanford University
  
  Started: 24 January 2005

  msh2tmf.cc - Utility to convert a MSH file to a TMF file

  Depends on:
  - Mesh class (mesh.hh)
  -- Vec3d class (vec3d.hh)
  -- Vert class (vert.hh)
  -- Edge class (edge.hh)
  -- Face class (face.hh)
  - Timer class (timer.hh)
  - TriMesh class (trimesh.h)
*/


#include <iostream>
#include "bmtk/mesh.hh"
#include "bmtk/timer.hh"
#include <trimesh/trimesh.h>

using namespace std;
using namespace bmtk;

int main (int argc, char *argv[]) {
  // if bad command line args, give usage instructions and exit
  if (argc < 3) {
    cout << endl 
	 << "Usage: msh2tmf infile.msh outfile.tmf" << endl
	 << "    infile.msh must be an msh file" << endl
	 << "    outfile.tmf is a tmf file" << endl;
    exit(1);
  }
  
  // read Mesh from file
  Mesh m;
  m.readMesh(argv[1]);

  // Export Mesh to ply format
  m.exportToTMF(argv[2]);
  
  return 0;
}
