/*
  James R. Diebel
  Stanford University

  Started: 18 January 2005

  off2msh.cc - Utility to convert a OFF file to a mesh (MSH) file

  Depends on:
  - Mesh class (mesh.hh)
  -- Vec3d class (vec3d.hh)
  -- Vert class (vert.hh)
  -- Edge class (edge.hh)
  -- Face class (face.hh)
  - Timer class (timer.hh)
  - TriMesh class (trimesh.h)
*/


#include <iostream>
#include "bmtk/mesh.hh"
#include "bmtk/timer.hh"
#include <trimesh/trimesh.h>

using namespace std;
using namespace bmtk;

int main (int argc, char *argv[]) {
  // if bad command line args, give usage instructions and exit
  if (argc < 3) {
    cout << endl
	 << "Usage: off2msh infile.off outfile.msh" << endl
	 << "    infile.off must be a .OFF file" << endl
	 << "    outfile.msh is the name of the output msh file" << endl;
    exit(1);
  }

  // create empty Mesh and then fill it with data from ply file
  Mesh m;
  Mesh* fm;
  m.buildFromOFF(argv[1]);
  //m.normalize();
  fm = m.pruneFaces(5);
  //fm->applyLineOfSightCovar(10);

  // output it to the specified file
  fm->writeMesh(argv[2]);
  //m.writeMesh(argv[2]);

  return 0;
}
