/*
  James R. Diebel
  Stanford University

  Started: 07 December 2004

  tmf2msh.cc - Utility to convert a TMF file to a mesh (MSH) file

  Depends on:
  - Mesh class (mesh.hh)
  -- Vec3d class (vec3d.hh)
  -- Vert class (vert.hh)
  -- Edge class (edge.hh)
  -- Face class (face.hh)
  - Timer class (timer.hh)
  - TriMesh class (trimesh.h)
*/


#include <iostream>
#include "bmtk/mesh.hh"
#include "bmtk/timer.hh"
#include <trimesh/trimesh.h>

using namespace std;
using namespace bmtk;

int main (int argc, char *argv[]) {
  // if bad command line args, give usage instructions and exit
  if (argc < 3) {
    cout << endl
	 << "Usage: tmf2msh infile.tmf outfile.msh" << endl
	 << "    infile.tmf must be a text mesh format file" << endl
	 << "    outfile.msh is the name of the output msh file" << endl;
    exit(1);
  }

  // create empty Mesh and then fill it with data from ply file
  Mesh m;
  m.buildFromTMF(argv[1]);
  m.applyLineOfSightCovar(1000.0f);


  // output it to the specified file
  m.writeMesh(argv[2]);

  return 0;
}
